<?php declare(strict_types=1);

namespace Sit\Creditpass\Components\CreditpassChecker;

use Symfony\Component\Validator\Constraints\DateTime;
use Shopware\Core\Content\Mail\Service\AbstractMailService;
use Shopware\Core\Framework\Uuid\Uuid;
use Shopware\Core\Framework\Validation\DataBag\DataBag;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Sit\Creditpass\Components\CreditpassChecker\CreditpassCheckerInterface;

class CreditpassChecker implements CreditpassCheckerInterface {

    /**
     * @var AbstractMailService
     */
    private $mailService;

    /**
     * @var SalesChannelContext
     */
    private $context;

    /**
     * @var int
     */
    private $_currentCheckResult = 0;

    /**
     * @var
     */
    private $configArray;

    /**
     * @var
     */
    private $checkRepository;

    /**
     * @param $requestArray
     * @param $configArray
     * @param $configurationsArray
     * @param $userDataArray
     * @param $checksArray
     * @param $context
     * @param $checkRepository
     * @param $mailService
     * @return array
     */
    public function processCreditpassCheck(
        $requestArray, $configArray, $configurationsArray, $userDataArray, $checksArray, $context, $checkRepository, $mailService
    ): array {

        $this->context          = $context;
        $this->mailService      = $mailService;
        $this->configArray      = $configArray;
        $this->checkRepository  = $checkRepository;

        $paymentId 			= $requestArray["paymentId"];
        $userGroupId		= $requestArray["userGroupId"];

        $paymentData = array();
        $paymentData['bankaccount'] 		= $requestArray['bankaccount'];
        $paymentData['bankaccountnumber'] 	= $requestArray['bankaccountnumber'];
        $paymentData['bankaccountiban'] 	= $requestArray['bankaccountiban'];
        $paymentData['creditcardtype'] 		= $requestArray['creditcardtype'];
        $paymentData['creditcardnumber'] 	= $requestArray['creditcardnumber'];
        $paymentData['amount']              = $requestArray['amount'];

        $this->sitCreditpassLog( '=> checkPaymentWithCreditpass()' );

// ### Standard module process from here ###

        // Check cp module is inactive
        if ( $this->configArray["sit_creditpass_settings_isactive"] != "1" ) {

            $this->sitCreditpassLog( '=> creditpass is inactive' );

            return array (
                'success' => 1,
                'message' => 'cp checks inactive',
            );
        }

        $this->sitCreditpassLog( '=> creditpass is active' );

        // Check secure payments
        $securePaymentSelection = $this->configArray["sit_creditpass_settings_secure_payments"];
        foreach ($securePaymentSelection as $securePaymentId) {

            if ($securePaymentId == $paymentId) {

                $this->sitCreditpassLog( '=> secure payment: ' . $securePaymentId );

                return array (
                    'success' => 1,
                    'message' => 'cp secure payment',
                );
            }
        }

        // Get creditpass configs
        $sitCreditpassConfigArray = $this->getMatchingCreditpassConfigs($userGroupId, $paymentId, $paymentData, $configurationsArray);

        // Check existing configs
        if ( empty($sitCreditpassConfigArray) ) {

            $this->sitCreditpassLog('=> no matching configs found');

            $missingConfigResult = $this->configArray["sit_creditpass_settings_missing_config_result"];
            if (strval($missingConfigResult) != '1') {
                $missingConfigResult = '0';
            }

            return array (
                'success' => $missingConfigResult,
                'message' => 'no matching config found',
            );
        }

        // Get maching config
        foreach ($sitCreditpassConfigArray as $sitCreditpassConfigRow) {

            $this->sitCreditpassLog('=> do creditpass check with config: ' . $sitCreditpassConfigRow["id"] );

            $message = $sitCreditpassConfigRow["sit_creditpass_config_errormessage"];

            // Do creditpass request
            $continue = $this->startCreditpassRequest($sitCreditpassConfigRow, $userGroupId, $paymentData, $userDataArray, $checksArray);
            if (!$continue) {
                $this->sitCreditpassLog( '<= check failed: ' . $sitCreditpassConfigRow["id"] );
                break;
            }

            $this->sitCreditpassLog( '<= check ok: ' . $sitCreditpassConfigRow["id"] );
        }

        // Set standard error message
        if ( empty($message) ) {
            $message = $this->configArray["sit_creditpass_settings_errormessage"];
        }

        $this->sitCreditpassLog( '<= creditpass check - finished' );

        // Create the return array
        return array (
            'success' => $this->_currentCheckResult,
            'message' => $message,
            'securepayments' => $securePaymentSelection
        );
    }

    /**
     * @param $userGroupId
     * @param $paymentId
     * @param $paymentData
     * @param $resultArrayConfig
     * @return array
     */
    private function getMatchingCreditpassConfigs($userGroupId, $paymentId, $paymentData, $resultArrayConfig): array {

        $this->sitCreditpassLog( '=> getMatchingCreditpassConfigs for payment: ' . $paymentId );

        // Return empty array if no configs given
        if ( count($resultArrayConfig) == 0 ) {
            $this->sitCreditpassLog( '<= getMatchingCreditpassConfigs no active configs found' );
            return array();
        }

        // Create matching config array
        $matchingConfigArray = array();

        // Run through configs
        foreach ($resultArrayConfig as $resultRowConfig) {

            $this->sitCreditpassLog( 'config check: ' . $resultRowConfig["sit_creditpass_config_name"] . ' (' . $resultRowConfig["id"] . ')' );

        // Check payments

            // Get payment-limitations
            $payments_selected = $resultRowConfig["sit_creditpass_config_payments"];

            // Check if current config has matching payment-limitation
            if (
                ! empty($payments_selected)
                && ! in_array($paymentId, $payments_selected)
            ) {

                $this->sitCreditpassLog( 'config not used due to payment-limitation' );
                continue;
            }

        // Check user groups

            // Get group-limitations
            $groups_selected = $resultRowConfig["sit_creditpass_config_groups"];

            // Check if current config has matching payment-limitation
            if (
                ! empty($groups_selected)
                && ! in_array($userGroupId, $groups_selected)
            ) {

                $this->sitCreditpassLog( 'config not used due to group-limitation' );
                continue;
            }

//            // Check shops
//            $shops_selected = $this->getConfigShopsSelected( $resultRowConfig["id"] );
//
//            // Check if current config has matching payment-limitation
//            if (
//                ! empty($shops_selected)
//                && ! in_array($sShopId, $shops_selected)
//            ) {
//                $this->sitCreditpassLog( 'config not used due to shop-limitation. current shop: ' . $sShopId );
//                continue;
//            }

            // Check payment
            $paymentAmount = $paymentData["amount"];
            if ( empty($paymentAmount) || $paymentAmount == "") {
                $paymentAmount = 0;
            }

            if ($paymentAmount < $resultRowConfig["sit_creditpass_config_paymentamount"]) {
                $this->sitCreditpassLog( 'config not used due to payment amount from' );
                continue;
            }
            if ($paymentAmount > $resultRowConfig["sit_creditpass_config_paymentamount_to"]) {
                $this->sitCreditpassLog( 'config not used due to payment amount to' );
                continue;
            }

            // Config is used
            $this->sitCreditpassLog( 'config used' );

            $matchingConfigArray[] =  $resultRowConfig;
        }

        $this->sitCreditpassLog( '<= getMatchingCreditpassConfigs' );

        return $matchingConfigArray;
    }

    /**
     * @param $sitCreditpassConfigRow
     * @param $userGroupId
     * @param $paymentData
     * @param $userDataArray
     * @param $checksArray
     * @return bool
     */
    private function startCreditpassRequest($sitCreditpassConfigRow, $userGroupId, $paymentData, $userDataArray, $checksArray): bool {

        $this->sitCreditpassLog('=> startCreditpassRequest');

    // Check whitelist

        // Get user groups
        $whitelist_groups_selected 	= $sitCreditpassConfigRow["sit_creditpass_config_wlgroups"];

        // Check if current config has matching payment-limitation
        if (
            ( ! empty($whitelist_groups_selected) )
            && in_array($userGroupId, $whitelist_groups_selected)
        ) {

            $this->sitCreditpassLog( 'result true (user in whitelist group): ' . $userGroupId);
            $this->_currentCheckResult = 1;
            return true;
        }

        // Get settings
        $mode = 8; // Test
        if ( $this->configArray["sit_creditpass_settings_istest"] == "0") {
            $mode = 1; // Live
        }

        $this->sitCreditpassLog('request mode: ' . $mode);

        $abortontimeout = true;
        if ($this->configArray["sit_creditpass_settings_abortontimeout"] == "0") {
            $abortontimeout = false;
        }

        $abortonmanualcheck = true;
        if ($this->configArray["sit_creditpass_settings_abortonmanualcheck"] == "0") {
            $abortonmanualcheck = false;
        }

        $abortonformaterror = true;
        if ($this->configArray["sit_creditpass_settings_abortonformaterror"] == "0") {
            $abortonformaterror = false;
        }

        $sendinfomailtoadmin = false;
        if ($this->configArray["sit_creditpass_settings_sendmailonerror"] == "1") {
            $sendinfomailtoadmin = true;
        }

        $sendmanualcheckmailtoadmin = false;
        if ($this->configArray["sit_creditpass_settings_sendmailonmanualcheck"] == "1") {
            $sendmanualcheckmailtoadmin = true;
        }

        // Get request data
        $url = $this->configArray["sit_creditpass_settings_url"];

        $user = $this->configArray["sit_creditpass_settings_user"];
        // Override standard user
        if ( ! empty($sitCreditpassConfigRow["sit_creditpass_config_user"]) ) {
            $user = $sitCreditpassConfigRow["sit_creditpass_config_user"];
        }

        $password = $this->configArray["sit_creditpass_settings_password"];
        // Override standard password
        if ( ! empty($sitCreditpassConfigRow["sit_creditpass_config_password"]) ) {
            $password = $sitCreditpassConfigRow["sit_creditpass_config_password"];
        }

        // Get repeat setting
        $repeatAfter = $sitCreditpassConfigRow["sit_creditpass_config_repeatcheck"];
        if ( empty($repeatAfter) ) {
            $repeatAfter = 9999999;
        }

        // Get abort setting
        $abortOnNegativCheck = false;
        if ($sitCreditpassConfigRow["sit_creditpass_config_abort_on_fail"] == "1") {
            $abortOnNegativCheck = true;
        }

        $code = $sitCreditpassConfigRow['sit_creditpass_config_purchase_type'];

    // Check user data

// echo "<pre>user:<br>";
// print_r($resultArrayUser);
// echo "</pre>";

        $email		= $userDataArray["username"];
        $company	= $userDataArray["company"];
        $phone		= $userDataArray["phone"];
        $firstname	= $userDataArray["firstname"];
        $lastname	= $userDataArray["lastname"];
        $birthdate  = $userDataArray["birthdate"];

        $gender		= ( strtoupper($userDataArray["salutation"]) == "MR" ? "M" : "") .
            ( strtoupper($userDataArray["salutation"]) == "MRS" ? "F" : "");

        $street		= $userDataArray["street"];
        $city		= $userDataArray["city"];
        $zip 		= $userDataArray["zip"];
        $country	= $userDataArray["countryiso"];

        // Check zip len for country
        $ziplen = 5; // Standard DE
        if ($country == "AT") {
            $ziplen = 4;
        }
        $zip	= ( strlen($zip) == $ziplen ? $zip : "");

    // Check payment data

// echo "<pre>payment:<br>";
// print_r($paymentData);
// echo "</pre>";

        $amount = $paymentData["amount"];
        $iban   = $paymentData["bankaccountiban"];
        $knr    = $paymentData["bankaccountnumber"];
        $blz    = $paymentData["bankaccount"];

        // Check if iban is given
        if (
            ( ! empty($iban) )
            && ( ! is_numeric($iban) )
        ) {
            $blz = "";
            $knr = "";
        }
        else {
            $iban = "";
        }

        // Check length of bankcode
        if ( strlen($blz) != 8 ) {
            $blz = "";
            $knr = "";
        }

        $kk_type = $paymentData["creditcardtype"];
        $kk_nr   = $paymentData["creditcardnumber"];
        if ( strlen($kk_nr) != 16 ) {
            $kk_nr = "";
            $kk_type = "";
        }

        // Create user data hash
        $userDataHash = md5($company . $phone . $email . $kk_type . $kk_nr . $knr . $blz . $iban . $country . $city . $zip . $street . $gender . $birthdate . $lastname . $firstname);

        $this->sitCreditpassLog('request - start check: ' . $sitCreditpassConfigRow['sit_creditpass_config_name'] );

        // Run through existing checks from DB
        $result = "";
        foreach ($checksArray as $resultRow) {

            $checkCode 			= $resultRow["code"];
            if ($checkCode != $code) {
                $this->sitCreditpassLog('request -> different purchase type -> skip');
                continue;
            }

            $checkDate 			= $resultRow["date"];
            $current_result		= $resultRow["result"];
            $request_hash		= $resultRow["request_hash"];

        // Check hash
            if ($userDataHash != $request_hash) {
                $this->sitCreditpassLog('request -> hashcode changed -> skip');
                continue;
            }

        // Check timespan since last request
            $dateDiff = time() - $checkDate->getTimestamp();
            $fullDays = floor($dateDiff/(60*60*24));

            // Check if check needs to be repeated
            if ($fullDays > $repeatAfter) {
                $this->sitCreditpassLog('request -> db result too old -> skip');
                continue;
            }

            $this->sitCreditpassLog('request -> result from db: ' . $current_result);

            $result = $current_result;

            break;
        }

        // Only check online if no existing result is given
        if ( $result == "" ) {

            $request_id = md5(uniqid("", TRUE));

            // Anonymize IP
            $ipAddress = $_SERVER['REMOTE_ADDR'];
            $ipAddressExplode = explode(".", $ipAddress);
            $ipAddressExplode[count($ipAddressExplode)-1] = "0";
            $ipAddress = implode(".", $ipAddressExplode);

            // Create request xml
            $request = '<?xml version="1.0" encoding="UTF-8" ?>' .
                '<REQUEST>' .
                '<CUSTOMER>' .
                '<AUTH_ID>' . $user . '</AUTH_ID>' .
                '<AUTH_PW>' . $password . '</AUTH_PW>' .
                '<CUSTOMER_TA_ID>' . $request_id . '</CUSTOMER_TA_ID>' .
                '</CUSTOMER>' .
                '<PROCESS>' .
                '<TA_TYPE>29070</TA_TYPE>' .
                '<PROCESSING_CODE>' . $mode . '</PROCESSING_CODE>' .
                '<REQUESTREASON>ABK</REQUESTREASON>' .
                '</PROCESS>' .
                '<QUERY>' .
                '<PURCHASE_TYPE>' . $code . '</PURCHASE_TYPE>' .
                '<AMOUNT>' . $amount . '</AMOUNT>' .
                '<FIRST_NAME>' . $this->replaceSpecialXMLChars($firstname) . '</FIRST_NAME>' .
                '<LAST_NAME>' . $this->replaceSpecialXMLChars($lastname) . '</LAST_NAME>' .
                '<DOB>' . $this->replaceSpecialXMLChars($birthdate) . '</DOB>' .
                '<GENDER>' . $gender . '</GENDER>' .
                '<ADDR_STREET_FULL>' . $this->replaceSpecialXMLChars($street) . '</ADDR_STREET_FULL>' .

                ( empty($zip) ? '' : '<ADDR_ZIP>' . $this->replaceSpecialXMLChars($zip) . '</ADDR_ZIP>') .

                '<ADDR_CITY>' . $this->replaceSpecialXMLChars($city) . '</ADDR_CITY>' .

                ( empty($country) ? '' : '<ADDR_COUNTRY>' . $this->replaceSpecialXMLChars($country) . '</ADDR_COUNTRY>') .

                '<BLZ>' . $this->replaceSpecialXMLChars($blz) . '</BLZ>' .
                '<KONTONR>' . $this->replaceSpecialXMLChars($knr) . '</KONTONR>' .
                '<IBAN>' . $this->replaceSpecialXMLChars($iban) . '</IBAN>' .
                '<PAN>' . substr($kk_nr, 0, 6) . '</PAN>' .
                '<PAN_TYPE>' . $kk_type . '</PAN_TYPE>' .
                '<CUSTOMERIP>' . $ipAddress . '</CUSTOMERIP>' .
                '<CUSTOMEREMAIL>' . $this->replaceSpecialXMLChars($email) . '</CUSTOMEREMAIL>' .
                '<CUSTOMERTEL>' . $this->replaceSpecialXMLChars($phone) . '</CUSTOMERTEL>' .
                '<COMPANY_NAME>' . $this->replaceSpecialXMLChars($company)  . '</COMPANY_NAME>' .
                '</QUERY>' .
                '</REQUEST>';

// echo "<pre>request:<br>";
// echo $request;
// echo "</pre>";

            $response = $this->postRequest($url, $request);

            $xml = simplexml_load_string($response['content']);
            $result = (String) $xml->PROCESS->ANSWER_CODE[0];

            $this->sitCreditpassLog( 'request -> result from creditpass: ' . $result );

            // Save result to db
            $this->saveCheckResult($userDataArray, $request, $userDataHash, $response['content'], $code, $result, $sitCreditpassConfigRow["id"] );
        }

        // Check for expected results
        // -1 = Format Error
        // 0 = Check positive
        // 1 = Check negative
        // 2 = Check manual
        if ( $result != "-1" && $result != "0" && $result != "1" && $result != "2" ) {
            $result = "unexpected"; // Unexpected answer (Timout or other error)
        }

        // Handle positive result
        if ($result == "0") {

            $this->sitCreditpassLog( 'request result true (check ok and continue)' );

            $this->_currentCheckResult = 1;

            return true;
        }

        // Handle format error result
        if ($result == "-1" && ! $abortonformaterror) {

            $this->sitCreditpassLog( 'request result true (format error and continue)' );

            $this->_currentCheckResult = 1;
            return true;
        }

        // Handle manual check mail
        if ( $result == "2"
            && $sendmanualcheckmailtoadmin
            && ! empty($this->configArray["sit_creditpass_settings_email"])
        ) {

            $this->sitCreditpassLog( 'request result manual check -> send info mail to admin' );

            $sSubject = "SIT-Creditpass-Check: Manuelle Prüfung für " . $firstname . " " . $lastname;

            // Create message
            $message  = "Kunde: " . $firstname . " " . $lastname;
            $message .= "<br>E-Mail: " . $email;
            $message .= "<br>Adresse: " . $street . " " . $zip . " " . $city;

            $this->sendMail($sSubject, $message, $this->configArray["sit_creditpass_settings_email"]);
        }

        // Handle manual check result
        if ($result == "2" && ! $abortonmanualcheck ) {

            $this->sitCreditpassLog( 'request result manual check -> continue true' );

            $this->_currentCheckResult = 1;
            return true;
        }

        // Handle unexpected result
        if ($result == "unexpected" && ! $abortontimeout) {

            $this->sitCreditpassLog( 'request result timeout -> continue true' );

            $this->_currentCheckResult = 1;
            return true;
        }

        $this->sitCreditpassLog( 'request result false' );

        // Negative result
        $this->_currentCheckResult = 0;

        // Send info email if needed
        if ( $sendinfomailtoadmin
            && ! empty($this->configArray["sit_creditpass_settings_email"])
        ) {

            $this->sitCreditpassLog( 'request result false -> send info mail to admin' );

            // Check for expected results
            $sSubject = "SIT-Creditpass-Check";

            // -1 = Format Error
            // 0 = Check positive
            // 1 = Check negative
            // 2 = Check manual
            if ($result == "-1") {
                $sSubject = "SIT-Creditpass-Check - negative Prüfung -> Formatfehler: " . $firstname . " " . $lastname;
            }
            else if ($result == "1") {
                $sSubject = "SIT-Creditpass-Check - negative Prüfung: " . $firstname . " " . $lastname;
            }
            else if ($result == "2") {
                $sSubject = "SIT-Creditpass-Check - negative Prüfung -> manuelle Prüfung: " . $firstname . " " . $lastname;
            }
            else if ($result == "unexpected") {
                $sSubject = "SIT-Creditpass-Check - negative Prüfung -> unerwartete Rückmeldung: " . $firstname . " " . $lastname;
            }

            // Create message
            $message  = "Kunde: " . $firstname . " " . $lastname;
            $message .= "<br>E-Mail: " . $email;
            $message .= "<br>Adresse: " . $street . " " . $zip . " " . $city;

            $this->sendMail($sSubject, $message, $this->configArray["sit_creditpass_settings_email"]);
        }

        if ($abortOnNegativCheck) {
            return false;
        }
        else {
            return true;
        }
    }

    private function replaceSpecialXMLChars($string): string {

        $string = str_replace("]", "", $string);
        $string = str_replace("[", "", $string);

        if (! empty($string) ) {
            $string = "<![CDATA[" . $this->encodeToUtf8( trim($string) ) . "]]>";
        }

        return $string;
    }

    private function encodeToUtf8($string): string {
        return mb_convert_encoding($string, "UTF-8", mb_detect_encoding($string, "UTF-8, ISO-8859-1, ISO-8859-15", true));
    }

    private function postRequest($sURL, $sMessage = ""): array {

        $postData = array();
        $postData["Reqpk"] = $sMessage;

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_URL, $sURL);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postData) );
        curl_setopt($ch, CURLOPT_HEADER, 1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, true);

        // Set curl options
        $response = curl_exec($ch);
        if ($response === FALSE) {

            $error = curl_error($ch);
            return array(
                'status' => 'err',
                'error' => $error
            );
        }

        // Get header and body
        $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $result = substr($response, $header_size);

        // Kein Fehler
        curl_close($ch);

        // return as structured array:
        return array(
            'status' => 'ok',
            'content' => $result
        );
    }

    // Shopware save
    private function saveCheckResult($userDataArray, $request, $userDataHash, $response, $purchaseType, $result, $configurationId) {

        $id = Uuid::randomHex();

        $this->checkRepository->create([
            [
                'id'                    => $id,
                'request'               => $request,
                'requestFirstname'      => $userDataArray["firstname"],
                'requestLastname'       => $userDataArray["lastname"],
                'requestStreet'         => $userDataArray["street"],
                'requestCity'           => $userDataArray["city"],
                'requestDate'           => date("Y-m-d H:i:s"),
                'requestPurchaseType'   => $purchaseType,
                'requestHash'           => $userDataHash,
                'response'              => $response,
                'responseResult'        => $result,
                'customerId'            => $userDataArray["id"],
                'configurationId'       => $configurationId
            ]
        ], $this->context->getContext());
    }

    /**
     * Shopware mailing
     * @param $subject
     * @param $message
     * @param $mailReceiver
     */
    private function sendMail($subject, $message, $mailReceiver) {

        $data = new DataBag();

        //basic e-mail data
        $data->set('recipients', [$mailReceiver => $mailReceiver]);
        $data->set('senderName', 'CreditpassCheck');
        $data->set('subject', $subject);
        $data->set('contentHtml', $message);
        $data->set('contentPlain', strip_tags($message));

        //set sales channel context
        $data->set('salesChannelId', $this->context->getSalesChannel()->getId());

        //send the e-mail
        $this->mailService->send($data->all(), $this->context->getContext(), []);
    }

    /**
     * @param $log
     */
    private function sitCreditpassLog($log) {

        // save notices to a logfile in a later Version
    }
}

