import template from './cp-configuration-detail.html.twig';
import './cp-configuration-detail.scss';

const { Component, Mixin } = Shopware;
const { EntityCollection, Criteria } = Shopware.Data;
const { mapPropertyErrors } = Shopware.Component.getComponentHelper();

Component.register('cp-configuration-detail', {
    template,

    inject: [
        'repositoryFactory'
    ],

    mixins: [
        Mixin.getByName('notification')
    ],

    data() {
        return {
            isLoading: null,
            isSaveSuccessful: false,
            cpConfiguration: null,
            categorySelections: {},
            paymentSelections: {},
            groupSelections: {},
            wlGroupSelections: {},
        };
    },

    metaInfo() {
        return {
            title: this.$createTitle()
        };
    },

    computed: {

        groupRepository() {

            return this.repositoryFactory.create("customer_group");
        },

        paymentRepository() {

            return this.repositoryFactory.create("payment_method");
        },

        categoryRepository() {

            return this.repositoryFactory.create("category");
        },

        repository() {
            return this.repositoryFactory.create('cp_configuration');
        },

        configurationCriteria() {

            const criteria = new Criteria();
            criteria.addAssociation('categories');
            criteria.addAssociation('payments');
            criteria.addAssociation('groups');
            criteria.addAssociation('wlgroups');

            return criteria;
        },

        ...mapPropertyErrors('cpConfiguration', [
            'amountFrom',
            'amountTo',
            'repeatAfter',
            'creditpassUser',
            'creditpassPassword',
            'purchaseType'
        ]),
    },

    watch: {
        '$route.params.id'() {
            this.createdComponent();
        }
    },

    created() {

        // Load empty selection if needed
        this.paymentSelections = new EntityCollection(
            this.paymentRepository.route,
            this.paymentRepository.entityName,
            Shopware.Context.api
        );

        // Load empty selection if needed
        this.groupSelections = new EntityCollection(
            this.groupRepository.route,
            this.groupRepository.entityName,
            Shopware.Context.api
        );

        // Load empty selection if needed
        this.categorySelections = new EntityCollection(
            this.categoryRepository.route,
            this.categoryRepository.entityName,
            Shopware.Context.api
        );

        // Load empty selection if needed
        this.wlGroupSelections = new EntityCollection(
            this.groupRepository.route,
            this.groupRepository.entityName,
            Shopware.Context.api
        );

        this.createdComponent();
    },

    methods: {

        createdComponent() {

            if (this.$route.params.id) {
                this.loadEntityData();
            }
            else {
                this.loadSelectionData();
            }
        },

        loadEntityData() {

            this.isLoading = true;

            this.repository.get(this.$route.params.id, Shopware.Context.api, this.configurationCriteria).then((entity) => {

                this.cpConfiguration = entity;
                this.isLoading = false;

                this.loadSelectionData();
            });
        },

        onSave() {

            this.isSaveSuccessful = false;

            this.onSaveUpdateCategorySelection();
            this.onSaveUpdatePaymentSelection();
            this.onSaveUpdateGroupSelection();
            this.onSaveUpdateGroupWhitelistSelection();

            this.repository.save(this.cpConfiguration, Shopware.Context.api).then(() => {

                this.isSaveSuccessful = true;
                this.onSaveCheckCredentials();

            }).catch((exception) => {

                this.isSaveSuccessful = false;

                this.createNotificationError({
                    title: this.$t('cp-trans.cp-config-detail.errorTitle'),
                    message: exception
                });
            });
        },

        onSaveCheckCredentials() {

            console.log("onSaveCheckCredentials");

            const headers = this.repository.buildHeaders(Shopware.Context.api);
            const httpClient = Shopware.Application.getContainer('init').httpClient;

            httpClient.get('/_action/sit-creditpass-credentials/' + this.cpConfiguration.creditpassUser + '/' + this.cpConfiguration.creditpassPassword, { headers }).then(
            (response) => {

                if (response.data.result != "0") {

                    console.log("errorCredentials: " + response.data);

                    this.createNotificationError({
                        title: this.$t('cp-trans.cp-config-detail.credentialErrorTitle'),
                        message: this.$t('cp-trans.cp-config-detail.credentialErrorMessage'),
                    });
                }
                else {
                    console.log("credentials ok");

                    this.createNotification({
                        title: this.$t('cp-trans.cp-config-detail.credentialSuccessTitle'),
                        message: this.$t('cp-trans.cp-config-detail.credentialSuccessMessage'),
                    });
                }
            });

        },

        loadCategories() {

            console.log("loadCategories");

            const criteria = new Criteria();

            if (this.cpConfiguration != null) {

                console.log("check cpConfiguration for categories");
                const categoryIds = this.cpConfiguration.categories.getIds();
                if (categoryIds != null && categoryIds.length > 0) {

                    console.log("categories found: " + categoryIds.length);
                    criteria.setIds(categoryIds);

                    this.categoryRepository.search(criteria, Shopware.Context.api).then((searchResult) => {
                        this.categorySelections = searchResult;
                    });

                    return;
                }
            }
        },

        loadPaymentMethods() {

            console.log("loadPaymentMethods");

            const criteria = new Criteria();
            if (this.cpConfiguration != null) {

                console.log("check cpConfiguration for payments");
                const paymentIds = this.cpConfiguration.payments.getIds();

                if (paymentIds != null && paymentIds.length > 0) {

                    console.log("payments found: " + paymentIds.length);
                    criteria.setIds(paymentIds);

                    this.paymentRepository.search(criteria, Shopware.Context.api).then((searchResult) => {
                        this.paymentSelections = searchResult;
                    });

                    return;
                }
            }
        },

        loadCustomerGroups() {

            console.log("loadCustomerGroups");

            const criteria = new Criteria();
            if (this.cpConfiguration != null) {

                console.log("check cpConfiguration for groups");
                const groupIds = this.cpConfiguration.groups.getIds();

                if (groupIds != null && groupIds.length > 0) {

                    console.log("groups found: " + groupIds.length);
                    criteria.setIds(groupIds);

                    this.groupRepository.search(criteria, Shopware.Context.api).then((searchResult) => {
                        this.groupSelections = searchResult;
                    });

                    return;
                }
            }
        },

        loadCustomerGroupsWhitelist() {

            console.log("loadCustomerGroupsWhitelist");

            const criteria = new Criteria();
            if (this.cpConfiguration != null) {

                console.log("check cpConfiguration for wlgroups");
                const groupIds = this.cpConfiguration.wlgroups.getIds();

                if (groupIds != null && groupIds.length > 0) {

                    console.log("wlgroups found: " + groupIds.length);
                    criteria.setIds(groupIds);

                    this.groupRepository.search(criteria, Shopware.Context.api).then((searchResult) => {
                        this.wlGroupSelections = searchResult;
                    });

                    return;
                }
            }
        },

        loadSelectionData() {

            console.log("loadSelectionData");

            this.loadCategories();
            this.loadPaymentMethods();
            this.loadCustomerGroups();
            this.loadCustomerGroupsWhitelist();

            if (this.$route.params.checkcredentials == true) {
                this.onSaveCheckCredentials();
            }
        },

        onSaveUpdateCategorySelection() {

            const categoryIds = this.cpConfiguration.categories.getIds();

            // Remove old categories
            categoryIds.forEach((categoryId) => {
                this.cpConfiguration.categories.remove(categoryId);
            });

            // Add new categories
            this.categorySelections.forEach((selection) => {
                this.cpConfiguration.categories.push(selection);
            });
        },

        onSaveUpdatePaymentSelection() {

            const paymentIds = this.cpConfiguration.payments.getIds();

            // Remove old payment
            paymentIds.forEach((paymentId) => {
                this.cpConfiguration.payments.remove(paymentId);
            });

            // Add new payments
            this.paymentSelections.forEach((selection) => {
                this.cpConfiguration.payments.push(selection);
            });
        },

        onSaveUpdateGroupSelection() {

            const groupIds = this.cpConfiguration.groups.getIds();

            // Remove old groups
            groupIds.forEach((groupId) => {
                this.cpConfiguration.groups.remove(groupId);
            });

            // Add new groups
            this.groupSelections.forEach((selection) => {
                this.cpConfiguration.groups.push(selection);
            });
        },

        onSaveUpdateGroupWhitelistSelection() {

            const groupIds = this.cpConfiguration.wlgroups.getIds();

            // Remove old groups
            groupIds.forEach((groupId) => {
                this.cpConfiguration.wlgroups.remove(groupId);
            });

            // Add new groups
            this.wlGroupSelections.forEach((selection) => {
                this.cpConfiguration.wlgroups.push(selection);
            });
        },

        onSaveFinish() {
            this.loadEntityData();
            this.isSaveSuccessful = true;
        },

        handleCategorySelectionChanged(selectedEntries) {
            this.categorySelections = selectedEntries;
        },

        handlePaymentSelectionChanged(selectedEntries) {
            this.paymentSelections = selectedEntries;
        },

        handleGroupSelectionChanged(selectedEntries) {
            this.groupSelections = selectedEntries;
        },

        handleWlGroupSelectionChanged(selectedEntries) {
            this.wlGroupSelections = selectedEntries;
        }
    }
});
