<?php declare(strict_types=1);

namespace Sit\Creditpass\Service;

use Shopware\Core\Checkout\Cart\SalesChannel\CartService;
use Shopware\Core\Content\Mail\Service\MailService;
use Shopware\Core\Defaults;
use Shopware\Core\Framework\Api\Context\SalesChannelApiSource;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\Validation\BuildValidationEvent;
use Shopware\Core\Framework\Validation\Exception\ConstraintViolationException;
use Shopware\Core\System\SalesChannel\Context\SalesChannelContextServiceInterface;
use Shopware\Core\System\SalesChannel\Context\SalesChannelContextServiceParameters;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Core\System\SystemConfig\SystemConfigService;
use Sit\Creditpass\Components\CreditpassChecker\CreditpassChecker;
use Sit\Creditpass\Core\Content\Configuration\ConfigurationEntity;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Validator\ConstraintViolation;
use Symfony\Component\Validator\ConstraintViolationInterface;
use Symfony\Component\Validator\ConstraintViolationList;


class CreditpassCheckoutValidator implements EventSubscriberInterface {

    public static function getSubscribedEvents() {

        return [
            'framework.validation.order.create' => 'onValidateOrder'
        ];
    }

    /**
     * @var $cartService
     */
    private $cartService;

    /**
     * @var ConfigurationEntity
     */
    private $configurationRepository;

    /**
     * @var $checkRepository
     */
    private $checkRepository;

    /**
     * @var $configService
     */
    private $configService;

    /**
     * @var $mailService
     */
    private $mailService;

    /**
     * @var RequestStack
     */
    private $requestStack;

    /**
     * @var SalesChannelContextServiceInterface
     */
    private $salesChannelContextService;

    /**
     * @param CartService $cartService
     * @param SystemConfigService $configService
     * @param EntityRepositoryInterface $configurationRepository
     * @param EntityRepositoryInterface $checkRepository
     * @param MailService $mailService
     * @param RequestStack $requestStack
     * @param SalesChannelContextServiceInterface $salesChannelContextService
     */
    public function __construct(
        CartService $cartService,
        SystemConfigService $configService,
        EntityRepositoryInterface $configurationRepository,
        EntityRepositoryInterface $checkRepository,
        MailService $mailService,
        RequestStack $requestStack,
        SalesChannelContextServiceInterface $salesChannelContextService
    ) {
        $this->cartService = $cartService;
        $this->configService = $configService;
        $this->configurationRepository = $configurationRepository;
        $this->checkRepository = $checkRepository;
        $this->mailService = $mailService;
        $this->requestStack = $requestStack;
        $this->salesChannelContextService = $salesChannelContextService;
    }

    /**
     * @param BuildValidationEvent $event
     */
    public function onValidateOrder(BuildValidationEvent $event) {

//        echo "onPaymentMethodChange: " . get_class($event);

        $context = $event->getContext();
        $source = $context->getSource();
        if (!$source instanceof SalesChannelApiSource) {
            return;
        }

        $salesChannelId = $source->getSalesChannelId();
        $token = $this->requestStack->getSession()->get('sw-context-token');

        $salesChannelContext = $this->salesChannelContextService->get(
            new SalesChannelContextServiceParameters($salesChannelId, $token, null, Defaults::CURRENCY)
        );

        $errorMessage = $this->checkPaymentsWithCreditpass($salesChannelContext);

        if ( $errorMessage != "" ) {

            if ( array_key_exists("testmode", $_REQUEST) && $_REQUEST["testmode"] == "1") {
                exit(0);
            }

            $data = $this->requestStack->getCurrentRequest()->request;

            $violationList = new ConstraintViolationList();
            $violationList->add(
                $this->buildViolation(
                    $errorMessage,
                    [''],
                    '/' . $errorMessage,
                    '',
                    "creditpass-error-message"
                )
            );

            // wirf diese Exception und du kommst zurück zur confirm seite
            throw new ConstraintViolationException($violationList, $data->all());
        }

        if ( array_key_exists("testmode", $_REQUEST) && $_REQUEST["testmode"] == "1") {
            exit(0);
        }
    }

    private function buildViolation(
        string $messageTemplate,
        array $parameters,
        string $propertyPath,
        string $invalidValue,
        string $code
    ): ConstraintViolationInterface {
        return new ConstraintViolation(
            str_replace(array_keys($parameters), array_values($parameters), $messageTemplate),
            $messageTemplate,
            $parameters,
            null,
            $propertyPath,
            $invalidValue,
            null,
            $code
        );
    }

    /**
     *
     */
    private function checkPaymentsWithCreditpass(SalesChannelContext $context): String {

        // Get user id
        $customer    = $context->getCustomer();
        $customerId  = $customer->getId();

        $customerGroup = $customer->getGroup();
        $customerGroupId = "";
        if ($customerGroup != null) {
            $customerGroupId = $customerGroup->getId();
        }

        // Get cart
        $cart = $this->cartService->getCart($context->getToken(), $context);
        $totalPrice = str_replace(".", "", strval($cart->getPrice()->getTotalPrice()) );

        // Get payment
        $paymentId = $context->getPaymentMethod()->getId();

        // Get basket article ids
//        $articleIdArray = $cart->getLineItems()->getReferenceIds();

    // Setup customer array
        $userDataArray = array();
        $userDataArray["id"]         = $customerId;
        $userDataArray["username"]   = $customer->getEmail();

        $userDataArray["birthdate"] = "";
        $birthday  = $customer->getBirthday();
        if ($birthday != null) {
            $userDataArray["birthdate"] = $birthday->format("Y-m-d");
        }

        $userDataArray["company"]    = $customer->getActiveBillingAddress()->getCompany();
        $userDataArray["phone"]      = $customer->getActiveBillingAddress()->getPhoneNumber();
        $userDataArray["firstname"]  = $customer->getActiveBillingAddress()->getFirstName();
        $userDataArray["lastname"]   = $customer->getActiveBillingAddress()->getLastName();

        $userDataArray["salutation"] = "";
        $salutation = $customer->getActiveBillingAddress()->getSalutation();
        if ($salutation != null) {
            $userDataArray["salutation"] = $salutation->getDisplayName();
        }

        $userDataArray["street"]     = $customer->getActiveBillingAddress()->getStreet();
        $userDataArray["city"]       = $customer->getActiveBillingAddress()->getCity();
        $userDataArray["zip"]        = $customer->getActiveBillingAddress()->getZipcode();
        $userDataArray["countryiso"] = $customer->getActiveBillingAddress()->getCountry()->getIso();

    // Setup request array
        $requestArray = array();
        $requestArray["paymentId"]          = $paymentId;
        $requestArray["userid"]             = $customerId;
        $requestArray["userGroupId"]        = $customerGroupId;
        $requestArray["bankaccount"]        = "";
        $requestArray["bankaccountnumber"]  = "";
        $requestArray["bankaccountiban"]    = "";
        $requestArray["creditcardtype"]     = "";
        $requestArray["creditcardnumber"]   = "";
        $requestArray["amount"]             = $totalPrice;

    // Setup config array
        $configArray = array();
        $configArray["sit_creditpass_settings_isactive"]                = $this->configService->get('SitCreditpass.config.isActive');
        $configArray["sit_creditpass_settings_istest"]                  = $this->configService->get('SitCreditpass.config.isTestmode');
        $configArray["sit_creditpass_settings_url"]                     = $this->configService->get('SitCreditpass.config.creditpassUrl');
        $configArray["sit_creditpass_settings_user"]                    = "";
        $configArray["sit_creditpass_settings_password"]                = "";
        $configArray["sit_creditpass_settings_email"]                   = $this->configService->get('SitCreditpass.config.creditpassEmail');
        $configArray["sit_creditpass_settings_errormessage"]            = $this->configService->get('SitCreditpass.config.standardErrormessage');
        $configArray["sit_creditpass_settings_sendmailonerror"]         = $this->configService->get('SitCreditpass.config.isSendMailOnNegativeCheck');
        $configArray["sit_creditpass_settings_sendmailonmanualcheck"]   = $this->configService->get('SitCreditpass.config.isSendMailOnManualCheck');
        $configArray["sit_creditpass_settings_missing_config_result"]   = $this->configService->get('SitCreditpass.config.isAllowPaymentOnMissingConfiguration');
        $configArray["sit_creditpass_settings_abortontimeout"]          = $this->configService->get('SitCreditpass.config.isAbortPaymentOnTimeout');
        $configArray["sit_creditpass_settings_abortonmanualcheck"]      = $this->configService->get('SitCreditpass.config.isAbortPaymentOnManualCheck');
        $configArray["sit_creditpass_settings_abortonformaterror"]      = $this->configService->get('SitCreditpass.config.isAbortPaymentOnFormatError');
        $configArray["sit_creditpass_settings_secure_payments"]         = $this->configService->get('SitCreditpass.config.creditpassSecurePayments');

    // Set configuration array
        $configurationsArray = array();

        $criteria = new Criteria();
        $criteria->addAssociation("groups");
        $criteria->addAssociation("wlgroups");
        $criteria->addAssociation("payments");
        $criteria->addAssociation("categories");

        $criteria->addFilter(new EqualsFilter('isActive', true));

        $configurations = $this->configurationRepository->search($criteria, $context->getContext());
        foreach($configurations as $configurationEntity) {

            $currentConfiguration = array();
            $currentConfiguration["id"]                                     = $configurationEntity->getId();
            $currentConfiguration["sit_creditpass_config_name"]             = $configurationEntity->getName();
            $currentConfiguration["sit_creditpass_config_paymentamount"]    = $configurationEntity->getAmountFrom();
            $currentConfiguration["sit_creditpass_config_paymentamount_to"] = $configurationEntity->getAmountTo();
            $currentConfiguration["sit_creditpass_config_repeatcheck"]      = $configurationEntity->getRepeatAfter();
            $currentConfiguration["sit_creditpass_config_errormessage"]     = $configurationEntity->getErrormessage();
            $currentConfiguration["sit_creditpass_config_user"]             = $configurationEntity->getCreditpassUser();
            $currentConfiguration["sit_creditpass_config_password"]         = $configurationEntity->getCreditpassPassword();
            $currentConfiguration["sit_creditpass_config_purchase_type"]    = $configurationEntity->getPurchaseType();
            $currentConfiguration["sit_creditpass_config_abort_on_fail"]    = $configurationEntity->getIsAbortOnFail();

            $currentConfiguration["sit_creditpass_config_groups"]           = $configurationEntity->getGroups()->getIds();
            $currentConfiguration["sit_creditpass_config_wlgroups"]         = $configurationEntity->getWlgroups()->getIds();
            $currentConfiguration["sit_creditpass_config_payments"]         = $configurationEntity->getPayments()->getIds();
            $currentConfiguration["sit_creditpass_config_categories"]       = $configurationEntity->getCategories()->getIds();

            $configurationsArray[] = $currentConfiguration;
        }

    // Setup check array
        $checksArray = array();

        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('customerId', $customerId));

        $checks = $this->checkRepository->search($criteria, $context->getContext());
        foreach($checks as $checkEntity) {

            $currentCheck = array();
            $currentCheck["code"]           = $checkEntity->getRequestPurchaseType();
            $currentCheck["date"]           = $checkEntity->getRequestDate();
            $currentCheck["request_hash"]   = $checkEntity->getRequestHash();
            $currentCheck["result"]         = $checkEntity->getResponseResult();

            $checksArray[] = $currentCheck;
        }

        $creditPassCheckerInterface = new CreditpassChecker();
        $returnArray = $creditPassCheckerInterface->processCreditpassCheck(
            $requestArray, $configArray, $configurationsArray, $userDataArray, $checksArray, $context,  $this->checkRepository, $this->mailService
        );

//        echo "<pre>return:<br>";
//            print_r($returnArray);
//        echo "</pre>";

        if ( $returnArray["success"] != 1 ) {

            $errorMessage = $this->encodeToUtf8( $returnArray["message"] );
            return $errorMessage;
        }

        return "";
    }

    private function encodeToUtf8( $string ): string {
        return mb_convert_encoding( $string, "UTF-8", mb_detect_encoding( $string, "UTF-8, ISO-8859-1, ISO-8859-15", true ) );
    }
}
